
import argparse
import json
import logging

from subunit import ByteStreamToStreamResult
from testtools import StreamToDict


#logging.basicConfig(level='INFO')
logger = logging.getLogger(__name__)

#logger.setLevel(logging.DEBUG)  # uncomment to enable debugging


class CreateJSONError(Exception):
    pass


class BuildResultDict(object):
    """Create a dictionary from a subunit stream file

    :param binary_stream: subunit file/stream to create dictionary from
    """
    def __init__(self, binary_stream):
        suite = self.get_stream_result(binary_stream)
        result = StreamToDict(self._handle_test)

        result.startTestRun()
        try:
            suite.run(result)
        except:
            pass
        finally:
            result.stopTestRun()

    def get_detail(self, detail_name):
        """Return a detail from the details dictionary

        :param detail_name: name of dictionary key to get values from
        """

        detail = self.details[detail_name]
        detail_type = detail.content_type.type
        if detail_type == 'text':
            return detail.as_text()
        else:
            # TODO? return new types when needed
            return detail_type

    def create_json(self):
        """create json format data to push to nfss"""
        result_dict = {}
        crash_list = []
        for keyname in self.details:
            if keyname.startswith('crash_file'):
                crash_list.append(self.get_detail(keyname))
                result_dict['crash_file'] = crash_list
            else:
                result_dict[keyname] = self.get_detail(keyname)
        json_dump = json.dumps(result_dict, indent=4)
        return json_dump

    def _handle_test(self, test_dict):
        """Call back method to get results from StreamToDict"""
        self.test_dict = test_dict
        self.details = test_dict['details']

    def get_stream_result(self, file_path):
        """Return stream result from byte stream"""
        logger.debug('opening file {}'.format(file_path))
        stream = open(file_path, 'rb')
        return ByteStreamToStreamResult(stream)

    def missing_key(self):
        keys = [
            'serial_number',
            'hardware_model',
            'total_test_time',
            'device_temp_during_crash',
            'ubuntu_version',
            'device_version',
            'free_mem',
            'cpu_useage',
            'battery_left',
            'release',
            'codename',
            'test_name',
        ]
        for key in keys:
            try:
                self.details[key]
            except AttributeError:
                logger.debug(
                    'could not find details in subunit stream'
                )
                return True
            except KeyError:
                logger.debug(
                    'could not find key {} in details'.format(key)
                )
                logger.debug('dictionary keys:')
                for k in self.details:
                    logger.debug(k)
                return True
        return False


def _create_parser():
    parser = argparse.ArgumentParser(
        description='create json file to upload to nfss from subunit file'
    )
    parser.add_argument(
        '-f',
        '--file',
        action='store',
        dest='subunit_file_path',
        required='true',
        help='path to subunit file',
    )
    parser.add_argument(
        '-v',
        '--verbose',
        action='store_true',
        dest='verbose',
        help='display debug messages while running',
    )

    return parser


if __name__ == '__main__':
    parser = _create_parser()
    args = parser.parse_args()
    if args.verbose:
        logger.setLevel(logging.DEBUG)
    file_path = args.subunit_file_path
    details_dict = BuildResultDict(file_path)
    json_data = details_dict.create_json()
    print(json_data)
