import argparse
import json
import logging
import numpy
import requests
import tempfile

from helpers_py2 import jenkins_pull, create_json_for_nfss, post_process_nfss


logging.basicConfig()
logger = logging.getLogger('post_process_jenkins_job')


class PostProcessJenkinsJobsError(Exception):
    pass


def check_no_image_in_nfss(project_name, test_name, image):
    nfss = post_process_nfss.NFSSAggregator(project_name)
    nfss_data = nfss.get_request(
        '{}/{}'.format(nfss.base_project_url, test_name)
    )
    if nfss_data.status_code is not 200:
        raise PostProcessJenkinsJobsError(
            'Did not get status code 200 back from nfss server got {}. '
            'check your internet connection or try again. use --init '
            'if this is your first time uploading data '.format(
                nfss_data.status_code
            )
        )
    nfss_json = nfss_data.json()
    if image in [data['data']['image'] for data in nfss_json['data']]:
        raise PostProcessJenkinsJobsError(
            'Image {} is already uploaded to nfss.'.format(image)
        )
    return True


def _create_parser():
    parser = argparse.ArgumentParser(
        description='create json file to upload to nfss from subunit file'
    )
    parser.add_argument(
        '-b',
        '--build',
        action='store',
        dest='jenkins_build_num',
        help='build number in jenkins to start pulling from',
    )
    parser.add_argument(
        '-i',
        '--image',
        action='store',
        dest='image',
        required='true',
        help='image to get data from',
    )
    parser.add_argument(
        '--init',
        action='store_true',
        dest='init',
        help='do this if it is the first time you run this script',
    )
    parser.add_argument(
        '-j',
        '--jenkins-job',
        action='store',
        dest='jenkins_job_name',
        required='true',
        help='jenkins job name to get data from',
    )
    parser.add_argument(
        '-p',
        '--project',
        action='store',
        dest='project_name',
        required='true',
        help='project name to get data from',
    )
    parser.add_argument(
        '-t',
        '--test',
        action='store',
        dest='test_name',
        required='true',
        help='test name to get data from',
    )
    parser.add_argument(
        '-v',
        '--verbose',
        action='store_true',
        dest='verbose',
        help='display debug messages while running',
    )

    return parser


class collect_data:
    def __init__(
        self,
        project_name,
        test_name,
        file_name,
        jenkins_job_name,
        image,
        last_build=13000
    ):
        """get subunit files for nfss

        :param project name: name of project to upload to nfss
        :param test_name: name of test to upload to nfss
        :param file_name: name of file to get from jenkins
        :param jenkins_job_name: name of file to get from jenkins
        :param image: name of image to get from jenkins
        """
        # last_build = get_last_build_number_from_nfss(project_name, test_name)
        self.image = image
        self.test_name = test_name
        pulled_list = jenkins_pull.url_artifact_list(
            jenkins_job_name,
            file_name,
            last_build,
        )

        time_list = []

        if pulled_list:
            pulled_list.sort()
            for build_info in pulled_list:
                if build_info['url'].endswith('subunit'):
                    url_file = build_info['url']
                else:
                    url_file = None
                if url_file:
                    self.download_file(url_file)
                    total_test_time = self._get_time_for_image()
                    self.subunit_file.close()
                    if (
                        total_test_time and
                        numpy.float(total_test_time) > 0 and
                        numpy.float(total_test_time) < 3660
                    ):
                        time_list.append(numpy.float(total_test_time))

        if time_list:
            avg = numpy.average(time_list)
            minimum = numpy.min(time_list)
            maximum = numpy.max(time_list)
            stddev = numpy.std(time_list)
            result = {
                'avg': avg,
                'min': minimum,
                'max': maximum,
                'stddev': stddev,
                'image': self.image,
                'build_num': self.build_num,
                'num_samples': len(time_list),
            }
            print json.dumps(result)

    def download_file(self, url):
        r = requests.get(url, stream=True)
        self.subunit_file = tempfile.NamedTemporaryFile()
        for chunk in r.iter_content(chunk_size=1024):
            if chunk:
                self.subunit_file.write(chunk)
                self.subunit_file.flush()
        return self.subunit_file

    def _get_time_for_image(self):
        subunit_result = create_json_for_nfss.BuildResultDict(
            self.subunit_file.name
        )
        if (
            'ubuntu_version' in subunit_result.details and
            'total_test_time' in subunit_result.details and
            'test_name' in subunit_result.details and
            'build_version' in subunit_result.details and
            subunit_result.get_detail('test_name') == self.test_name and
            str(subunit_result.get_detail('ubuntu_version')) == str(self.image)
        ):
            self.build_num = subunit_result.get_detail('build_version')
            return subunit_result.get_detail('total_test_time')
        return False


if __name__ == '__main__':
    parser = _create_parser()
    args = parser.parse_args()
    if args.jenkins_build_num:
        build_num = args.jenkins_build_num
    else:
        build_num = 0
    if args.verbose:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.ERROR)
    jenkins_job_name = args.jenkins_job_name
    project_name = args.project_name
    test_name = args.test_name
    image = args.image
    # don't upload the image more than once, don't check the first time
    if not args.init:
        check_no_image_in_nfss(project_name, test_name, image)

    collect_data(
        project_name,
        test_name,
        'result.subunit',
        jenkins_job_name,
        image,
        build_num,
    )
