import json
import requests
import logging
import settings

logger = logging.getLogger(__name__)


class LRTNFSSPreProcessorError(Exception):
    pass


class NFSSAggregator:
    def __init__(self, project_name):
        self.nfss_base_url = settings.NFSS_URL
        logger.debug('nfss base url: {}'.format(self.nfss_base_url))
        self.base_api_url = '{}/api/v1'.format(self.nfss_base_url)
        logger.debug('nfss api url: {}'.format(self.base_api_url))
        self.base_project_url = '{}/{}'.format(
            self.base_api_url,
            project_name
        )
        logger.debug('base project api url: {}'.format(self.base_project_url))

    def get_request(self, url):
        """Get request from url

        :param url: url to get from
        """
        request = requests.get(url)
        if request.status_code == 200:
            return request
        if request.status_code == 500:
            return False

        raise LRTNFSSPreProcessorError(
            'unexpected response from url: '.format(url)
        )

    def format_request(self, request, to_dict=True):
        """Format request to dictionary

        :param request: request from get_request
        """
        if to_dict:
            return request.json()
        else:
            return json.loads(request.json())

    def _get_test_list(self, url):
        """Return all tests as a list"""
        request = self.get_request(url)
        json_data = self.format_request(request)
        return json_data['tests']

    def get_all_raw_test_list(self):
        """Return only raw tests as a list (no aggregated tests)"""
        tests = self._get_test_list(self.base_project_url)
        return [test for test in tests if not test.endswith('aggregate')]

    def get_raw_data_from_tests_for_image(self, test_name_list, image='all'):
        image_data = []
        for test_name in test_name_list:
            request = self.get_request(
                "{}/{}".format(self.base_project_url, test_name)
            )
            request_data = self.format_request(request)
            if image == 'all':
                image_data.append(
                    [
                        data for data in request_data['data']
                    ]
                )

            else:
                image_data.append(
                    [
                        data for data in request_data['data'] if
                        'ubuntu_version' in str(data['data']) and
                        'total_test_time' in str(data['data']) and
                        str(data['data']['ubuntu_version']) == str(image)
                    ]
                )
        return image_data

    def get_latest_image_version(self, json_data):
        """Get latest version in NFSS

        :param json_data: Data to get last version from
        """
        return json_data['data'][-1]['data']['ubuntu_version']

    def aggregate_daily_image_all_tests(self, url, image_number):
        pass

    def aggregate_daily_image_sepecific_tests(self, url, test_name):
        pass

if __name__ == '__main__':
    a = NFSSAggregator('lrt')
    l = a.get_all_raw_test_list()
    l = ['random_gestures']
    d = a.get_raw_data_from_tests_for_image(l, 'all')
    time_list = []
    from decimal import Decimal
    for i in d:
        for ii in i:
            if 'total_test_time' in ii['data']:
                if(
                    Decimal(ii['data']['total_test_time']) > 0 and
                    Decimal(ii['data']['total_test_time']) < 4000
                ):
                    time_list.append(
                        int(Decimal(ii['data']['total_test_time']))
                    )
    import numpy
    print(numpy.average(time_list))
