
"""lrt package.

This root package contains utilities for all the LRT tests.

"""

import subprocess
import time

from testtools import TestCase
from testtools.content import (
    text_content,
)

from lrt.gestures import(
    unlock_ui,
    get_display_size
)


def _run_command(command, get_returncode=False):
    """run a command using subprocess_check_output"""
    process = subprocess.Popen(
        command,
        shell=True,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        close_fds=True
    )
    if get_returncode:
        process.communicate()
        return process.returncode

    return process.communicate()


def take_screenshot(file_name):
    """
    :param file_name: name to call screenshot will save as file_name.png
    """
    display_size = get_display_size()
    _run_command(
        'mirscreencast -n 1 -m /run/mir_socket -f {}.rgba'.format(file_name)
    )
    _run_command(
        'convert -depth 8 -size {} {}.rgba {}.png'.format(
            display_size,
            file_name,
            file_name
        )
    )


def compare_screenshots(file1, file2):
    """
    :param file1: the first file to compare
    :param file2: the second file to compare
    """
    output = _run_command(
        'compare -metric PSNR {} {} compared.png'.format(file1, file2)
    )[0].strip().decode('utf8')
    if output == 'inf':
        return True
    return False


def _get_prop(prop):
    """Get property from getprop command"""
    return _run_command('getprop {}'.format(prop))[0].strip().decode('utf8')


def _get_cpu_usage():
    """percentage of cpu being used"""
    return _run_command(
        "top -bn1 | awk '/Cpu/ { print 100 - $8 }'"
    )[0].strip().decode('utf8')


def _get_mem_free():
    """free memory left"""
    return _run_command(
        "free -h | grep Mem | awk '{print $4}'"
    )[0].strip().decode('utf8')


def _get_battery_power():
    """get battery power"""
    return _run_command(
        "cat /sys/class/power_supply/battery/capacity"
    )[0].strip().decode('utf8')


def _get_ubuntu_release():
    """get the ubuntu release version"""
    return _run_command(
        'lsb_release -r | cut -f2'
    )[0].strip().decode('utf8')


def _get_ubuntu_codename():
    """get the ubuntu release version"""
    return _run_command(
        'lsb_release -c | cut -f2'
    )[0].strip().decode('utf8')


def get_crash_logs():
    """Returns a set of .crash files on the system."""
    return set(
        _run_command(
            'sudo ls /var/crash | grep crash'
        )[0].decode('utf8').splitlines()
    )


def get_file_permissions(file_path):
    return str(
        _run_command(
            'sudo stat -c "%a %n" {}'.format(file_path)
        )
    )[0].split(' ')[0]


def wait_for_crash_file_permissions(file_path):
    while int(get_file_permissions(file_path)) <= 0:
        time.sleep(2)
    return True


class LRTTestCase(TestCase):

    def setUp(self):
        super().setUp()
        self.have_frozen = False
        self.have_screenshot = False
        serial_number = _get_prop('ro.boot.serialno')
        hardware_model = _get_prop('ro.boot.hardware')
        codename = _get_ubuntu_codename()
        release = _get_ubuntu_release()
        self.addDetail(
            'serial_number',
            text_content(serial_number)
        )
        self.addDetail(
            'hardware_model',
            text_content(hardware_model)
        )
        self.addDetail(
            'codename',
            text_content(codename)
        )
        self.addDetail(
            'release',
            text_content(release)
        )

        with open('/etc/system-image/channel.ini') as ini_file:
            for line in ini_file:
                if 'version_detail' in line:
                    for version in line.split(':')[1].split(','):
                        if 'ubuntu' in version:
                            self.addDetail(
                                'ubuntu_version',
                                text_content(
                                    str(version.strip()).split('=')[1]
                                )
                            )
                        elif 'device' in version:
                            self.addDetail(
                                'device_version',
                                text_content(
                                    str(version.strip()).split('=')[1]
                                )
                            )
                        else:
                            self.addDetail(
                                'build_version',
                                text_content(
                                    str(version.strip()).split('=')[1]
                                )
                            )

        # turn off the modem, to prevent us from calling emergency services:
        _run_command(
            '/usr/share/ofono/scripts/offline-modem',
            get_returncode=True
        )

        self._existing_crash_files = get_crash_logs()
        self._start_time = time.clock_gettime(time.CLOCK_MONOTONIC_RAW)
        self._screenshot_clock = time.clock_gettime(time.CLOCK_MONOTONIC_RAW)
        self._desired_runtime = 43200  # run for 12 hours
        unlock_ui()  # open greeter

    def tearDown(self):
        super().tearDown()
        # self.addDetail('temperature_data', self.device_temp)

    def check_for_crash(self):
        """Check to see if a crash has occured.

        If a crash hasn't occured, this method will return control to the test
        being run. If a test has occured, the crash file will be attached to
        the test result stream, and the test will be failed. In this case, the
        time to failure will be included in the assertion.

        """
        have_crashed = False

        if self.have_frozen:
            take_screenshot("three")
            same = compare_screenshots("first.png", "three.png")
            if same:
                self.have_crashed = True
            else:
                self.have_frozen = False

        if self.get_current_screenshot_clock() >= 240:
            take_screenshot("last")
            same = compare_screenshots("first.png", "last.png")
            if same:
                self.have_frozen = True
            self._screenshot_clock = time.clock_gettime(
                time.CLOCK_MONOTONIC_RAW
            )
            self.have_screenshot = False
        elif(
            self.have_screenshot is False and
            self.get_current_screenshot_clock() >= 120
        ):
            take_screenshot("first")
            self.have_screenshot = True

        for crash_file in get_crash_logs() - self._existing_crash_files:
            crash_path = '/var/crash/{}'.format(crash_file)
            # Make sure the it's a crash and not a recoverable error

            if (
                _run_command(
                    'sudo head {} | grep "ProblemType: Crash"'.format(
                        crash_path
                    ), get_returncode=True) == 0
            ):
                have_crashed = True
                break

        if self.get_current_run_time() >= self._desired_runtime:
            self.addDetail(
                'total_test_time',
                text_content(str(self.get_current_run_time()))
            )
            raise TestCase.skipException("Test run completed.")

        if have_crashed:
            crash_time = self.get_current_run_time()
            self.addDetail(
                'device_temp_during_crash',
                text_content(self.get_current_temperature())
            )
            self.addDetail(
                'test_was_frozen',
                text_content(str(self.have_frozen))
            )
            self.addDetail(
                'cpu_useage',
                text_content(_get_cpu_usage())
            )
            self.addDetail(
                'free_mem',
                text_content(_get_mem_free())
            )
            self.addDetail(
                'battery_left',
                text_content(_get_battery_power())
            )
            self.addDetail(
                'total_test_time',
                text_content(str(crash_time))
            )
            for crash_file in list(get_crash_logs()):
                crash_path = '/var/crash/{}'.format(crash_file)
                self.addDetailUniqueName(
                    'crash_file',
                    text_content(crash_path)
                )

            if self.have_frozen:
                raise AssertionError(
                    "UI hung after %d seconds" % crash_time
                )
            raise AssertionError(
                "Something crashed after %d seconds." %
                crash_time
            )

    def get_current_temperature(self):
        return _run_command(
            'sudo cat /sys/devices/virtual/thermal/thermal_zone10/temp'
        )[0].strip().decode('utf8')

    def get_current_screenshot_clock(self):
        current_time = time.clock_gettime(time.CLOCK_MONOTONIC_RAW)
        return current_time - self._screenshot_clock

    def get_current_run_time(self):
        current_time = time.clock_gettime(time.CLOCK_MONOTONIC_RAW)
        return current_time - self._start_time
